/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005-6 Aidan Thornton <makomk@lycos.co.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifndef NGCS_COMMON_H
#define NGCS_COMMON_H
#include "../../initng-paths.h"

#define SOCKET_ROOTPATH      DEVDIR "/initng/"
#define SOCKET_FILENAME_REAL SOCKET_ROOTPATH "initng.ngcs"
#define SOCKET_FILENAME_TEST SOCKET_ROOTPATH "initng-test.ngcs"

/*! \brief Null type
 *
 *  Describes a lack of data. The associated data is ignored and should be empty.
 */
#define NGCS_TYPE_NULL 0

/*! \brief Native integer
 *
 *  This represents an int. The associated data is an int in native byte order.
 */
#define NGCS_TYPE_INT 1

/*! \brief Counted string
 *
 *  Represents a string (not neccesarily null-terminated). The actual data is 
 *  the contents of the string.
 */
#define NGCS_TYPE_STRING 2


/*! \brief Blob
 *
 * Just some chunk of data - no interpretation is specified.
 */
#define NGCS_TYPE_BLOB 3


/*! \brief Native long
 *
 *  The associated data is a long in native byte order.
 */
#define NGCS_TYPE_LONG 4


/*! \brief Compound data structure
 *
 *  The associated data is a data structure. It consists of a sequence of 
 *  entries, each made up of two ints (native byte order) corresponding to the
 *  data type and length of data, followed by the actual data.
 */
#define NGCS_TYPE_STRUCT 6

typedef struct ngcs_data_s
{
    int type;
    int len;
    union
    {
        int i;
        long l;
        char *s;
        void *p;
    } d;
} ngcs_data;

int ngcs_sendall(int sock, const void *buf, int len);
int ngcs_recvall(int sock, void *buf, int len);

int ngcs_sendmsg(int sock, int chan, int type, int len, char *data);

int ngcs_recvmsg(int sock, int *chan, int *type, int *len, char **data);

int ngcs_pack(ngcs_data * data, int cnt, char *buf);
int ngcs_unpack(const char *data, int len, ngcs_data ** res);
void ngcs_free_unpack(int len, ngcs_data * res);
#endif
